"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var sinon = require("sinon");
var httpLink_1 = require("../src/httpLink");
var Links = require("apollo-link-core");
var apollo_link_core_1 = require("apollo-link-core");
var apollo_fetch_1 = require("apollo-fetch");
var graphql_1 = require("graphql");
var graphql_tag_1 = require("graphql-tag");
var fetchMock = require("fetch-mock");
var sampleQuery = (_a = ["\n  query SampleQuery {\n    stub {\n      id\n    }\n  }\n"], _a.raw = ["\n  query SampleQuery {\n    stub {\n      id\n    }\n  }\n"], graphql_tag_1.default(_a));
var sampleMutation = (_b = ["\n  mutation SampleMutation {\n    stub(param: \"value\") {\n      id\n    }\n  }\n"], _b.raw = ["\n  mutation SampleMutation {\n    stub(param: \"value\") {\n      id\n    }\n  }\n"], graphql_tag_1.default(_b));
describe('HttpLink', function () {
    var data = { hello: 'world', method: 'POST' };
    var mockError = { throws: new TypeError('mock me') };
    var subscriber;
    beforeEach(function () {
        fetchMock.post('begin:data', data);
        fetchMock.post('begin:error', mockError);
        var next = sinon.spy();
        var error = sinon.spy();
        var complete = sinon.spy();
        subscriber = {
            next: next,
            error: error,
            complete: complete,
        };
    });
    afterEach(function () {
        fetchMock.restore();
    });
    it('raises warning if called with concat', function () {
        var link = apollo_link_core_1.ApolloLink.from([new httpLink_1.default()]);
        var _warn = console.warn;
        console.warn = function (warning) { return chai_1.assert.property(warning, 'message'); };
        chai_1.assert.deepEqual(link.concat(function (operation, forward) { return forward(operation); }), link);
        console.warn = _warn;
    });
    it('does not need any constructor arguments', function () {
        chai_1.assert.doesNotThrow(function () { return new httpLink_1.default(); });
    });
    it('calls next and then complete', function (done) {
        var next = sinon.spy();
        var link = new httpLink_1.default({ uri: 'data' });
        var observable = apollo_link_core_1.execute(link, {
            query: sampleQuery,
        });
        observable.subscribe({
            next: next,
            error: function (error) { return chai_1.assert(false); },
            complete: function () {
                chai_1.assert(next.calledOnce);
                done();
            },
        });
    });
    it('calls error when fetch fails', function (done) {
        var link = new httpLink_1.default({ uri: 'error' });
        var observable = apollo_link_core_1.execute(link, {
            query: sampleQuery,
        });
        observable.subscribe(function (result) { return chai_1.assert(false); }, function (error) {
            chai_1.assert.equal(error, mockError.throws);
            done();
        }, function () {
            chai_1.assert(false);
            done();
        });
    });
    it('calls error when fetch fails', function (done) {
        var link = new httpLink_1.default({ uri: 'error' });
        var observable = apollo_link_core_1.execute(link, {
            query: sampleMutation,
        });
        observable.subscribe(function (result) { return chai_1.assert(false); }, function (error) {
            chai_1.assert.equal(error, mockError.throws);
            done();
        }, function () {
            chai_1.assert(false);
            done();
        });
    });
    it('unsubscribes without calling subscriber', function (done) {
        var link = new httpLink_1.default({ uri: 'data' });
        var observable = apollo_link_core_1.execute(link, {
            query: sampleQuery,
        });
        var subscription = observable.subscribe(function () { return chai_1.assert(false); }, function () { return chai_1.assert(false); }, function () { return chai_1.assert(false); });
        subscription.unsubscribe();
        chai_1.assert(subscription.closed);
        setTimeout(done, 50);
    });
    var verifyRequest = function (link, after) {
        var next = sinon.spy();
        var context = { info: 'stub' };
        var variables = { params: 'stub' };
        var observable = apollo_link_core_1.execute(link, {
            query: sampleMutation,
            context: context,
            variables: variables,
        });
        observable.subscribe({
            next: next,
            error: function (error) { return chai_1.assert(false); },
            complete: function () {
                var body = JSON.parse(fetchMock.lastCall()[1].body);
                chai_1.assert.equal(body.query, graphql_1.print(sampleMutation));
                chai_1.assert.deepEqual(body.context, context);
                chai_1.assert.deepEqual(body.variables, variables);
                chai_1.assert.equal(next.callCount, 1);
                after();
            },
        });
    };
    it('passes all arguments to multiple fetch body', function (done) {
        var link = new httpLink_1.default({ uri: 'data' });
        verifyRequest(link, function () { return verifyRequest(link, done); });
    });
    it('calls multiple subscribers', function (done) {
        var link = new httpLink_1.default({ uri: 'data' });
        var context = { info: 'stub' };
        var variables = { params: 'stub' };
        var observable = apollo_link_core_1.execute(link, {
            query: sampleMutation,
            context: context,
            variables: variables,
        });
        observable.subscribe(subscriber);
        observable.subscribe(subscriber);
        setTimeout(function () {
            chai_1.assert(subscriber.next.calledTwice);
            chai_1.assert(subscriber.error.notCalled);
            chai_1.assert(subscriber.complete.calledTwice);
            done();
        }, 50);
    });
    it('calls remaining subscribers after unsubscribe', function (done) {
        var link = new httpLink_1.default({ uri: 'data' });
        var context = { info: 'stub' };
        var variables = { params: 'stub' };
        var observable = apollo_link_core_1.execute(link, {
            query: sampleMutation,
            context: context,
            variables: variables,
        });
        observable.subscribe(subscriber);
        var subscription = observable.subscribe(subscriber);
        subscription.unsubscribe();
        setTimeout(function () {
            chai_1.assert(subscriber.next.calledOnce);
            chai_1.assert(subscriber.error.notCalled);
            chai_1.assert(subscriber.complete.calledOnce);
            done();
        }, 50);
    });
    it('should add headers from the context', function (done) {
        var fetch = apollo_fetch_1.createApolloFetch({
            customFetch: function (request, options) {
                return new Promise(function (resolve, reject) {
                    chai_1.assert.property(options.headers, 'test');
                    chai_1.assert.deepEqual(options.headers.test, context.headers.test);
                    done();
                });
            },
        });
        var link = new httpLink_1.default({ fetch: fetch });
        var context = {
            headers: {
                test: 'header',
            },
        };
        Links.execute(link, { query: sampleQuery, context: context }).subscribe(chai_1.expect.fail);
    });
});
var _a, _b;
//# sourceMappingURL=httpLink.js.map